class Installer::Action
  def initialize(args)
    @data = {}
    @status = :initialized
  end

  def go(&update_block)
    raise NotImplementedError, "Action must implement a go method"
  end

  def ssh_update_proc(&stream_block)
    Proc.new do |ch, stream, data|
      if stream == :stdout
        severity = :info
      elsif stream == :stderr
        severity = :warn
      else
        raise NotImplementedError, "Unknown stream #{stream}: #{data}"
      end

      ssh_progress_event = Installer::Event.new(:action_progress, self.name)
      message = Installer::Event::Message.new(severity, ch.connection.host, data)
      ssh_progress_event.add_message(message)
      stream_block.call(ssh_progress_event)
    end
  end

  def success?
    false
  end

  def name
    raise NotImplementedError, "Action must have a name"
  end

  def description
    raise NotImplementedError, "Action must have a description"
  end
end

require 'installer/action/upload'
require 'installer/action/execute'
require 'installer/action/resolve_dns'
