require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::HardwareCheck < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super
  end

  def perform_step(&stream_block)
    @infra.hosts.each do |hostname, host|
      cpuinfo = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /proc/cpuinfo")
      yield cpuinfo

      processors = 0
      if cpuinfo.success?
        cpuinfo.results[hostname][:stdout].each_line do |l|
          processors += 1 if /[Pp]rocessor\s+:/.match(l)
        end
      else
        warning(hostname, "Error reading CPU config: Could not execute 'cat /proc/cpuinfo'.")
      end

      meminfo = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /proc/meminfo")
      yield meminfo

      totalram = 0
      if meminfo.success?
        stdout = meminfo.results[hostname][:stdout]
        meminfo.results[hostname][:stdout].each_line do |l|
          memtotal = /MemTotal:\s+(\d+) kB/.match(l)
          totalram = memtotal[1].to_i if memtotal
        end
        totalram /= 1024
      else
        warning(hostname, "Error reading memory config: Could not execute 'cat /proc/meminfo'.")
      end

      if processors < 4
        cores = processors == 1 ? 'core' : 'cores'
        warning(hostname, "We found #{processors} CPU #{cores}. We recommend at least 4 cores.")
      end

      if totalram < 4096
        warning(hostname, "We found #{totalram} MB RAM. We recommend at least 4096 MB.")
      end
    end
  end

  def name
    :hardware_check
  end

  def description
    "Verifying that your hardware meets requirements on #{@infra.hosts.keys.join(',')}."
  end
end
